@@ -463,12 +463,14 @@ eval {require 'OraSpriteFns.pl';};
 use vars qw ($VERSION $LOCK_SH $LOCK_EX);
 ##--
 
-$JSprite::VERSION = '5.22';
+$JSprite::VERSION = '5.24';
 $JSprite::LOCK_SH = 1;
 $JSprite::LOCK_EX = 2;
 
 my $NUMERICTYPES = '^(NUMBER|FLOAT|DOUBLE|INT|INTEGER|NUM)$';       #20000224
 my $STRINGTYPES = '^(VARCHAR2|CHAR|VARCHAR|DATE|LONG|BLOB|MEMO)$';
+my @perlconds = ();
+my @perlmatches = ();
 
 ##++
 ##  Public Methods and Constructor
@@ -519,7 +521,8 @@ sub new
 		silent       => 0,
 		dirty			 => 0,     #JWT: 20000229: PREVENT NEEDLESS RECOMMITS.
 		StrictCharComp => 0,    #JWT: 20010313: FORCES USER TO PAD STRING LITERALS W/SPACES IF COMPARING WITH "CHAR" TYPES.
-		sprite_forcereplace => 0  #JWT: 20010912: FORCE DELETE/REPLACE OF DATAFILE (FOR INTERNAL WEBFARM USE)!
+		sprite_forcereplace => 0,  #JWT: 20010912: FORCE DELETE/REPLACE OF DATAFILE (FOR INTERNAL WEBFARM USE)!
+		dbuser			=> ''      #JWT: 20011026: SAVE USER'S NAME.
 	    };
 
     $self->{separator} = { Unix  => '/',    Mac => ':',   #JWT: BUGFIX.
@@ -535,6 +538,7 @@ sub initialize
 {
     my $self = shift;
 
+    $sprite_user = $self->{'dbuser'};   #ADDED 20011026.
     $self->define_errors;
     $self->set_os ($^O) if (defined $^O);
 }
@@ -697,11 +701,11 @@ sub unlock
 sub sql
 {
     my ($self, $query) = @_;
-
     my ($command, $status);
 
     return wantarray ? () : -514  unless ($query);
 
+	$sprite_user = $self->{'dbuser'};   #ADDED 20011026.
 	$self->{lasterror} = 0;
 	$self->{lastmsg} = '';
     $query   =~ s/\n/ /gs;
@@ -778,7 +782,6 @@ sub commit
     $status = 1;
     return $status  unless ($self->{dirty});
 
-#print "<BR>Sprite::Commit: file=$file= FILE=".$self->{file}."=\n";
 	if ($file)
 	{
 		$full_path = $self->get_path_info ($file);
@@ -787,9 +790,7 @@ sub commit
 	else   #ADDED 20010911 TO ASSIST IN HANDLING AUTOCOMMIT!
 	{
 		$full_path = $self->{file};
-#print "<BR>Sprite::Commit: using SPRITE FILE=".$self->{file}."= STATUS=$status=\n";
 	}
-#print "<BR>!!! COMMIT WRITING FILE =$full_path=\n";
 	$status = $self->write_file ($full_path);
 	$self->display_error ($status) if ($status <= 0);
 
@@ -841,6 +842,7 @@ sub define_errors
     $errors->{'-520'} = "Can not create existing table (drop first!).";  #20000225 JWT.
     $errors->{'-521'} = "Can not change datatype on non-empty table.";  #20000323 JWT.
     $errors->{'-522'} = "Can not decrease field-size on non-empty table.";  #20000323 JWT.
+    $errors->{'-523'} = "Special table \"DUAL\" is READONLY!";  #20000323 JWT.
 
     $self->{errors} = $errors;
 
@@ -850,7 +852,6 @@ sub define_errors
 sub parse_expression
 {
     my ($self, $query) = @_;
-#print STDERR "-PARSE_EXPRESSION: at 1 query=$query=\n";
     return unless ($query);
     my ($column, @strings, %numopmap, %stropmap, $numops, $strops, $special);
 	my ($colmlist) = join('|',@{$self->{order}});
@@ -897,14 +898,17 @@ $query =~ s/\\\\/\x02/g;    #PROTECT "\\"
 $query =~ s/\\\'|\'\'/\x03/g;   #20000201  #PROTECT "", \", '', AND \'.
 #$query =~ s/\\\"|\"\"/\x04/g;   #REMOVED 20000303.
 
-#print STDERR "-2: query=$query=\n";
-
 my ($i, $j, $j2, $k);
 while (1)
 {
 	$i = 0;
-	$i = ($query =~ s|(\w)\s+not\s+like\s+|$1 !^ |i);
-	$i = ($query =~ s|(\w)\s+like\s+|$1 =^ |i)  unless ($i);
+	#$i = ($query =~ s|(\w)\s+not\s+like\s+|$1 !^ |i);
+	#$i = ($query =~ s|(\w)\s+like\s+|$1 =^ |i)  unless ($i);
+	#20011017: LAST 2 CHGD. TO NEXT 2 TO (PARTIALLY) FIX BUG FORBIDDING THE WORD 'LIKE' IN LITERAL STRINGS IN WHERE CLAUSES.
+	#$i = ($query =~ s|\b($column)\s+not\s+like\s+|$1 !^ |i);
+	#$i = ($query =~ s|\b($column)\s+like\s+|$1 =^ |i)  unless ($i);
+	$i = ($query =~ s|\b($colmlist)\s+not\s+like\s+|$1 !^ |i);
+	$i = ($query =~ s|\b($colmlist)\s+like\s+|$1 =^ |i)  unless ($i);
 	if ($i)
 	{
 		#if ($query =~ /(\^\s*["'])([^"\\]*(\\.[^"\\]*)*)/)
@@ -930,7 +934,6 @@ while (1)
 		last;
 	}
 }
-#print STDERR "-3: query=$query=\n";
 	
     #$query =~ s/([!=][~\^])\s*(m)?([^\w;\s])([^\3\\]*(?:\\.[^\3\\]*)*)\3(i)?/
 
@@ -955,16 +958,30 @@ while (1)
 	|e);
 
 	#THIS REGEX HANDLES ALL OTHER LIKE AND PERL "=~" AND "!~" OPERATORS.
-#print STDERR "-4: query=$query=\n";
 
-    $query =~ s%([!=][~\^])\s*(m)?(.)([^\3]*?)\3(i)?%
-	           my ($m, $i, $delim, $four, $one) = ($2, $5, $3, $4, $1);
+	@perlconds = ();
+    #$query =~ s%([!=][~\^])\s*(m)?(.)([^\3]*?)\3(i)?%  #20011017: CHGD TO NEXT.
+
+    $query =~ s%\b($colmlist)\s*([!=][~\^])\s*(m)?(.)([^\4]*?)\4(i)?%  #20011017: CHGD TO NEXT.
+	           my ($m, $i, $delim, $four, $one, $fldname) = ($3, $6, $4, $5, $2, $1);
+	           my ($catchmatch) = 0;
                    $m ||= ''; $i ||= '';
 					$m = 'm'  unless ($delim eq '/');
 					my ($three) = $delim;
+					$four =~ s/\\\(/\x02/g;
+					$four =~ s/\\\)/\x03/g;
+					if ($four =~ /\(.*\)/)
+					{
+						#$four =~ s/\(//g;
+						#$four =~ s/\)//g;
+						$catchmatch = 1;
+					}
+					$four =~ s/\x02/\(/g;
+					$four =~ s/\x03/\)/g;
                     push (@strings, "$m$delim$four$three$i");
-                   "$one *$#strings";
-               %ge;
+					push (@perlconds, "\$_->{$fldname} $one *$#strings; push (\@perlmatches, \$1)  if (defined \$1); push (\@perlmatches, \$2)  if (defined \$2);")  if ($catchmatch);
+                   "$fldname $one *$#strings";
+               %gei;
     #$query =~ s|(['"])([^\1\\]*(?:\\.[^\1\\]*)*)\1|
     $query =~ s|(["'])(.*?)\1|
                    push (@strings, "$1$2$1"); "*$#strings";
@@ -973,7 +990,6 @@ while (1)
 	$query =~ s/\x03/\'/g;   #RESTORE PROTECTED SINGLE QUOTES HERE.
 	#$query =~ s/\x04/\"/g;   #RESTORE PROTECTED DOUBLE QUOTES HERE.   #REMOVED 20000303.
 	$query =~ s/\x02/\\/g;   #RESTORE PROTECTED SLATS HERE.
-#print STDERR "-6: query=$query=\n";
 
 	for $i (0..$#strings)
 	{
@@ -985,53 +1001,42 @@ while (1)
 	if ($query =~ /^($column)$/)
 	{
 		$i = $1;
-		$query = '&' . $i  unless ($i =~ $colmlist);
+		#$query = '&' . $i  unless ($i =~ $colmlist);  #CHGD. TO NEXT (20011019)
+		$query = '&' . $i  unless ($i =~ m/($colmlist)/i);
 	}
 
-#print STDERR "-15: query=$query=\n";
-    $query =~ s#\b($column)\s*($numops)\s*\*#$1 $numopmap{$2} \*#go;
-    $query =~ s#\b($column)\s*($numops)\s*\'#$1 $numopmap{$2} \'\'#go;
-#print STDERR "-16: query=$query=\n";
-    $query =~ s#\b($column)\s*($numops)\s*($colmlist)#$1 $numopmap{$2} $3#go;
-    #$query =~ s#\b($column)\s*($numops)\s*($column(?:\(.*?\))?)#$1 $numopmap{$2} $3#go;
+    $query =~ s#\b($colmlist)\s*($numops)\s*\*#$1 $numopmap{$2} \*#gi;
+    $query =~ s#\b($colmlist)\s*($numops)\s*\'#$1 $numopmap{$2} \'\'#gi;
+    $query =~ s#\b($colmlist)\s*($numops)\s*($colmlist)#$1 $numopmap{$2} $3#gi;
+    #$query =~ s#\b($colmlist)\s*($numops)\s*($column(?:\(.*?\))?)#$1 $numopmap{$2} $3#gi;
     $query =~ s%\b($column\s*(?:\(.*?\))?)\s+is\s+null%$1 eq ''%ig;
-#print STDERR "-17: query=$query=\n";
     $query =~ s%\b($column\s*(?:\(.*?\))?)\s+is\s+not\s+null%$1 ne ''%ig;
-    #$query =~ s%\b($column)\s*(?:\(.*?\))?)\s*($numops)\s*CURVAL%$1 $2 &pscolfn($self,$3)%g;
+    #$query =~ s%\b($colmlist)\s*(?:\(.*?\))?)\s*($numops)\s*CURVAL%$1 $2 &pscolfn($self,$3)%gi;
     $query =~ s%($column)\s*($numops)\s*($column\.(?:$psuedocols))%"$1 $2 ".&pscolfn($self,$3)%eg;
-#print STDERR "-19: query=$query=\n";
     $query =~ s%\b($column\s*(?:\(.*?\))?)\s*($numops)\s*($column\s*(?:\(.*?\))?)%
 		my ($one,$two,$three) = ($1,$2,$3);
 		$one =~ s/\s+$//;
-#print STDERR "-20a: 1=$one= 2=$two= 3=$three=\n";
 		if ($one =~ /NUM\s*\(/ || ${$self->{types}}{"\U$one\E"} =~ /$NUMERICTYPES/i)
 		{
 			$two =~ s/^($strops)$/$stropmap{$two}/;
-#print STDERR "-20N: 1=$one= 2=$two= 3=$three= tp=".${$self->{types}}{"\U$one\E"};
-#print "-20N: 1=$one= 2=$two= 3=$three= tp=".${$self->{types}}{"\U$one\E"};
 			"$one $two $three";
 		}
 		else
 		{
-#print "-20C: 1=$1= 2=$2= 3=$3= tp=".${$self->{types}}{"\U$one\E"};
-#print STDERR "-20C: 1=$1= 2=$2= 3=$3= tp=".${$self->{types}}{"\U$one\E"};
 			"$one $numopmap{$two} $three";
 		}
 	 %eg;
-#print STDERR "-21A: query=$query=\n";
 
-# (JWT 8/8/1998) $query =~ s|\b($column)\s+($strops)\s+(\d+)|$1 $stropmap{$2} $3|gio;
-	$query =~ s|\b($column)\s*($strops)\s*(\d+)|$1 $stropmap{$2} $3|gio;
-#print STDERR "-21B: query=$query=\n";
+# (JWT 8/8/1998) $query =~ s|\b($colmlist)\s+($strops)\s+(\d+)|$1 $stropmap{$2} $3|gi;
+	$query =~ s|\b($colmlist)\s*($strops)\s*(\d+)|$1 $stropmap{$2} $3|gi;
 
 	my $ineqop = '!=';
-	$query =~ s!\b($column)\s*($strops)\s*(\*\d+)!
+	$query =~ s!\b($colmlist)\s*($strops)\s*(\*\d+)!
 		my ($one,$two,$three) = ($1,$2,$3);
 		$one =~ s/\s+$//;
 		my $res;
 		if ($one =~ /NUM\s*\(/ || ${$self->{types}}{"\U$one\E"} =~ /$NUMERICTYPES/i)
 		{
-#print STDERR "\n-21N: BEF: 1=$one= 2=$two= 3=$three= tp=".${$self->{types}}{"\U$one\E"};
 			my ($opno) = undef;    #NEXT 18 LINES ADDED 20010313 TO CAUSE STRING COMPARISENS W/NUMERIC FIELDS TO RETURN ZERO, SINCE PERL NON-NUMERIC STRINGS RETURN ZERO.
 			if ($three =~ /^\*\d+/)
 			{
@@ -1044,19 +1049,16 @@ while (1)
 			{
 				$opno = $three;
 			}
-#print STDERR "\n-21Nc: opno=$opno=\n";
 			unless ($opno =~ /^[\+\-\d\.][\d\.Ex\+\-\_]*$/)  #ARGUMENT IS A VALID NUMBER.
 			{
 			#	$res = '0';
 			#	$res = '1'  if ($two eq $ineqop);
-#print STDERR "\n-21Nd: AFT: 1=$one= 2=$two= 3=$three= opno=$opno=\n";
 				$res = "$one $two '0'";
 			}
 			else
 			{
 				$two =~ s/^($strops)$/$stropmap{$two}/  unless ($opno eq "0");
 				$res = "$one $two $three";
-#print STDERR "\n-21Ne: AFT: 1=$one= 2=$two= 3=$three= opno=$opno=\n";
 			}
 		}
 		elsif ($self->{StrictCharComp} == 0 && ${$self->{types}}{"\U$one\E"} eq 'CHAR')
@@ -1072,24 +1074,20 @@ while (1)
 							'%-'.${$self->{lengths}}{"\U$one\E"}.'s',
 							$opstr) . "'";
 			}
-#print STDERR "-21C: 1=$one= 2=$two= 3=$three= tp=".${$self->{types}}{"\U$one\E"};
 			$res = "$one $two $three";
 		}
 		else
 		{
-#print STDERR "-21V: 1=$one= 2=$two= 3=$three= tp=".${$self->{types}}{"\U$one\E"};
 			$res = "$one $two $three";
 		}
 		$res;
-		!eg;
+		!egi;
 
-#print STDERR "-22 query=$query=\n";
 	#NOTE!:  NEVER USE ANY VALUE OF $special AS A COLUMN NAME IN YOUR TABLES!!
 	#20000224 ADDED "\b" AFTER "$special)" 5 LINES BELOW!
 	$query =~ s!\b(($colmlist))\b!
                    my $match = $1;
 						$match =~ tr/a-z/A-Z/;
-#print "-23: MATCH=$match=\n";
                    ($match =~ /\b(?:$special)\b/io) ? "\L$match\E"    : 
                                                     "\$_->{$match}"
                !gei;
@@ -1100,9 +1098,14 @@ while (1)
 
 	$query =~ s|(\d+)\s*($strops)\s*(\d+)|$1 $stropmap{$2} $3|gio;   #ADDED 20010313 TO MAKE "1=0" CONDITION EVAL WO/ERROR.
     $query =~ s|\*(\d+)|$strings[$1]|g;
+	 for (my $i=0;$i<=$#perlconds;$i++)
+	 {
+	 	$perlconds[$i] =~ s|\*(\d+)|$strings[$1]|g;
+	 }
+
     
 	#THIS REGEX EVALS USER-FUNCTION CALLS FOLLOWING "=~" OR "!~".
-#print "-25: query=$query=\n";
+
 	$query =~ s@([!=][~\^])\s*m\&([a-zA-Z_]+[^&]*)\&@
 			my ($one, $two) = ($1, $2);
 			
@@ -1154,7 +1157,7 @@ sub parse_columns
 			$ordercols, $descorder, $fields, $distinct) = @_;
     my ($i, $j, $k, $rowcnt, $status, @columns, $single, $loop, $code, $column);
 	my (%colorder, $rawvalue);
-#print "-parse_columns: at=".join('|',@_)."= distinct=$distinct=\n";
+	my ($psuedocols) = "CURVAL|NEXTVAL";   #ADDED 20011019.
 	local $results = undef;
 
 	my (@keyfields) = split(',', $self->{key_fields});  #JWT: PREVENT DUP. KEYS.
@@ -1185,17 +1188,14 @@ $| = 1;
 
 	my (@these_results);
 	$fieldregex = $self->{fieldregex};
+	my ($skipreformat) = 0;
+	my ($colskipreformat) = 0;
     for ($loop=0; $loop < scalar @{ $self->{records} }; $loop++)
 	{
 		next unless (defined $self->{records}->[$loop]);    #JWT: DON'T RETURN BLANK DELETED RECORDS.
 		$_ = $self->{records}->[$loop];
-#foreach my $xxx (keys(%{$_})) {print STDERR "-value($xxx) =$_->{$xxx}=\n";};
-#print STDERR "->>>>> condition=$condition=\n";
-#print "<BR> condition=$condition=\n";
 		$@ = '';
 		if ( !$condition || (eval $condition) ) {
-#print "<BR>-- CONDITION RETURNED TRUE! res=$@=\n";
-#print STDERR "-!!!!!- CONDITION RETURNED TRUE!\n";
 		    if ($command eq 'select')
 		    {
 				if ($fields)
@@ -1203,6 +1203,7 @@ $| = 1;
 					@these_results = ();
 					for (my $i=0;$i<=$#{$fields};$i++)
 					{
+						$fields->[$i] =~ s/($self->{column}\.(?:$psuedocols))\b/&pscolfn($self,$1)/eg;  #ADDED 20011019
 						push (@these_results, eval $fields->[$i]);
 					}
 					push (@$results, [ @these_results ]);
@@ -1213,6 +1214,11 @@ $| = 1;
 				}
 		    } 
 		    elsif ($command eq 'update') {
+		    @perlmatches = ();
+		    for (my $i=0;$i<=$#perlconds;$i++)
+		    {
+		    		eval $perlconds[$i];
+		    }
 			$code = '';
 			my ($matchcnt) = 0;
 			my (@valuelist) = keys(%$values);
@@ -1253,13 +1259,18 @@ NOMATCHED1:
 			$self->{dirty} = 1;
 			foreach $jj (@columns)  #JWT 19991104: FORCE TRUNCATION TO FIT!
 			{
-				$rawvalue = $values->{$jj};
+				$colskipreformat = $skipreformat;
+				#$rawvalue = $values->{$jj};  #CHGD TO NEXT 20011018.
+				$rawvalue = $valuenames{$jj};
+				#NEXT LINE ADDED 20011018 TO HANDLE PERL REGEX SUBSTITUTIONS.
+				$colskipreformat = 0  if ($rawvalue =~ s/\$(\d)/$perlmatches[$1-1]/g);
 				#if ($rawvalue =~ /^[_a-zA-Z]/)  #NEXT 5 LINES ADDED 20000516 SO FUNCTIONS WILL WORK IN UPDATES!
 				if ($valuenames{$jj} =~ /^[_a-zA-Z]/)  #NEXT 5 LINES ADDED 20000516 SO FUNCTIONS WILL WORK IN UPDATES!
 				{
-					unless ($self->{fields}->{$valuenames{$jj}})  #ADDED TEST 20001218 TO ALLOW FIELD-NAMES AS RIGHT-VALUES.
+					unless ($self->{fields}->{"\U$valuenames{$jj}\E"})  #ADDED TEST 20001218 TO ALLOW FIELD-NAMES AS RIGHT-VALUES.
 					{
-						$rawvalue = &chkcolumnparms($valuenames{$jj});
+						#$rawvalue = &chkcolumnparms($valuenames{$jj}); #CHGD. TO NEXT 20011018.
+						$rawvalue = &chkcolumnparms($rawvalue);
 						$rawvalue = eval $rawvalue;   #FUNCTION EVAL 3
 						return (-517)  if ($@);
 					}
@@ -1267,48 +1278,53 @@ NOMATCHED1:
 					{
 						$rawvalue = $_->{$valuenames{$jj}};
 					}
+					$colskipreformat = 0;
 				}
 				else
 				{
-					$rawvalue =~ s/^\'(.*)\'\s*$/$1/  if ($values->{$jj} =~ /^\'/);
+					$rawvalue =~ s/^\'(.*)\'\s*$/$1/  if ($valuenames{$jj} =~ /^\'/);
 				}
 				#if (${$self->{types}}{$jj} =~ /$NUMERICTYPES/)  #CHGD TO NEXT LINE 20010313.
-				if (length($rawvalue) > 0 && ${$self->{types}}{$jj} =~ /$NUMERICTYPES/)
+
+				unless ($colskipreformat)   #ADDED 20011018 TO OPTIMIZE.
 				{
-					$k = sprintf(('%.'.${$self->{scales}}{$jj}.'f'), 
-							$rawvalue);
-				}
-				else
-				{
-					$k = $rawvalue;
-				}
-				$rawvalue = substr($k,0,${$self->{lengths}}{$jj});
-				unless ($self->{LongTruncOk} || $rawvalue eq $k || 
-						(${$self->{types}}{$jj} eq 'FLOAT'))
-				{
-					$errdetails = "$jj to ${$self->{lengths}}{$jj} chars";
-					return (-519);   #20000921: ADDED (MANY PLACES) LENGTH TO ERRDETAILS "(fieldname to ## chars)"
-				}
-				if ((${$self->{types}}{$jj} eq 'FLOAT') 
-						&& (int($rawvalue) != int($k)))
-				{
-					$errdetails = "$jj to ${$self->{lengths}}{$jj} chars";
-					return (-519);
-				}
-				if (${$self->{types}}{$jj} eq 'CHAR')
-				{
-					$values->{$jj} = "'" . sprintf(
-							'%-'.${$self->{lengths}}{$jj}.'s',
-							$rawvalue) . "'";
-				}
-				#elsif (${$self->{types}}{$jj} !~ /$NUMERICTYPES/)  #CHGD. TO NEXT 20010313.
-				elsif (!length($rawvalue) || ${$self->{types}}{$jj} !~ /$NUMERICTYPES/)
-				{
-					$values->{$jj} = "'" . $rawvalue . "'";
-				}
-				else
-				{
-					$values->{$jj} = $rawvalue;
+					if (length($rawvalue) > 0 && ${$self->{types}}{$jj} =~ /$NUMERICTYPES/)
+					{
+						$k = sprintf(('%.'.${$self->{scales}}{$jj}.'f'), 
+								$rawvalue);
+					}
+					else
+					{
+						$k = $rawvalue;
+					}
+					$rawvalue = substr($k,0,${$self->{lengths}}{$jj});
+					unless ($self->{LongTruncOk} || $rawvalue eq $k || 
+							(${$self->{types}}{$jj} eq 'FLOAT'))
+					{
+						$errdetails = "$jj to ${$self->{lengths}}{$jj} chars";
+						return (-519);   #20000921: ADDED (MANY PLACES) LENGTH TO ERRDETAILS "(fieldname to ## chars)"
+					}
+					if ((${$self->{types}}{$jj} eq 'FLOAT') 
+							&& (int($rawvalue) != int($k)))
+					{
+						$errdetails = "$jj to ${$self->{lengths}}{$jj} chars";
+						return (-519);
+					}
+					if (${$self->{types}}{$jj} eq 'CHAR')
+					{
+						$values->{$jj} = "'" . sprintf(
+								'%-'.${$self->{lengths}}{$jj}.'s',
+								$rawvalue) . "'";
+					}
+					#elsif (${$self->{types}}{$jj} !~ /$NUMERICTYPES/)  #CHGD. TO NEXT 20010313.
+					elsif (!length($rawvalue) || ${$self->{types}}{$jj} !~ /$NUMERICTYPES/)
+					{
+						$values->{$jj} = "'" . $rawvalue . "'";
+					}
+					else
+					{
+						$values->{$jj} = $rawvalue;
+					}
 				}
 			}
 			map { $code .= qq|\$_->{'$_'} = $values->{$_};| } @columns;
@@ -1321,11 +1337,11 @@ NOMATCHED1:
 			delete $_->{$single};
 		    }
 			++$rowcnt;
+			$skipreformat = 1;
 		}
 		elsif ($@)   #ADDED 20010313 TO CATCH SYNTAX ERRORS.
 		{
 			$errdetails = "Condition failed ($@) in condition=$condition!";
-#print STDERR "\n-!!!!!!!!!!! CONDITION FAILED, AT=$@=\n";
 			return -503  if ($command eq 'select');
 			return -505  if ($command eq 'delete');
 			return -504;
@@ -1340,7 +1356,6 @@ NOMATCHED1:
 	{
 		return $rowcnt;
     } else {
-#print "-???- dist=$distinct=\n";
 		if ($distinct)   #THIS IF ADDED 20010521 TO MAKE "DISTINCT" WORK.
 		{
 			my (%disthash);
@@ -1352,7 +1367,6 @@ NOMATCHED1:
 			foreach my $i (keys(%disthash))
 			{
 				push (@$results, [split(/\x02/, $i)]);
-#print "-results=$i=\n";			
 			}
 		}
 		if (@$ordercols)
@@ -1398,8 +1412,8 @@ NOMATCHED1:
 ####select * from medm_users where fn like 'j%' order by (ln,fn) desc
 					++$k;
 				}
-				$SA[$i] .= '|' . $_;
-				++$i;
+				$SA[$i] .= '|' . $_;  #NOTE: "|" DOES *NOT* NEED TO BE PROTECTED!
+				++$i;                 #(ONLY THE *LAST* VALUE OF THE SPLIT IS USED).
 			}
 			@SSA = sort {$a cmp $b} @SA;  #SORT 'EM!
 			@SSA = reverse(@SSA)  if ($descorder);
@@ -1501,27 +1515,28 @@ sub select
     my ($i, @l, $regex, $path, $columns, $table, $extra, $condition, 
 			$values_or_error);
 	my (@ordercols) = ();
-#print STDERR "-select: query=$query=\n";
-#print STDERR "-select: query=$query=\n";
     $regex = $self->{_select};
     $path  = $self->{path};
-$fieldregex = $self->{fieldregex};
+	$fieldregex = $self->{fieldregex};
 
 	my $distinct;   #NEXT 2 ADDED 20010521 TO ADD "DISTINCT" CAPABILITY!
 	$distinct = 1  if ($query =~ /^select\s+distinct/);
 	$query =~ s/^select\s+distinct(\s+\w|\s*\(|\s+\*)/select $1/i;
 
-	#HANDLE SELECTS WITH JUST FIELD NAMES (NO FUNCTIONS) FAST.
 
-	if ($query =~ /^select\s+                         # Keyword
-                    ($regex)\s+                       # Columns
-                    from\s+                           # 'from'
-                    ($path)(.*)$/iox)
-	{           
-		($columns, $table, $extra) = ($1, $2, $3);
-#print STDERR "-select, at 2: field names only, extra=$extra=\n";
-		#if ($columns =~ /^table_name$/i && $table =~ /^user_tables$/i)  #JWT: FETCH TABLE NAMES! - CHANGED TO NEXT LINE 20010301 TO FIX BUG.
-		if ($columns =~ /^table_name\s*$/i && $table =~ /^user_tables$/i)  #JWT: FETCH TABLE NAMES!
+
+    if  ($query =~ /^select\s+
+			(.+)\s+
+			from\s+
+			(\w+)(.*)$/iox)
+    {
+		my ($column_stuff, $table, $extra) = ($1, $2, $3);
+    		my (@fields) = ();
+    		my ($fnname, $found_parin, $parincnt);
+
+		#ORACLE COMPATABILITY!
+
+		if ($column_stuff =~ /^table_name\s*$/i && $table =~ /^user_tables$/i)  #JWT: FETCH TABLE NAMES!
 		{
 			$full_path = $self->{directory};
 			$full_path .= $self->{separator}->{ $self->{platform} }  
@@ -1574,47 +1589,6 @@ END_CODE
 			unshift (@$values_or_error, ($#l+1));
 			return $values_or_error;
 		}
-#print STDERR "-select: 2a\n";
-		$thefid = $table;
-		$self->check_for_reload ($table) || return (-501);
-		if ($extra =~ s/([\s|\)]+)order\s+by\s*(.*)/$1/i)
-		{
-			$orderclause = $2;
-			@ordercols = split(/,/,$orderclause);
-			$descorder = ($ordercols[$#ordercols] =~ s/(\w+\W+)desc(?:end|ending)?$/$1/i);  #MODIFIED 20000721 TO ALLOW "desc|descend|descending"!
-			#$orderclause =~ s/,\s+/,/g;
-			for $i (0..$#ordercols)
-			{
-				$ordercols[$i] =~ s/\s//g;
-				$ordercols[$i] =~ s/[\(\)]+//g;
-			}
-		}
-#print STDERR "-select, at 2c: extra=$extra=\n";
-		if ($extra =~ /^\s+where\s*(.+)$/i)
-		{
-#print STDERR "-select, at 2d: extra=$1=\n";
-		
-		    $condition = $self->parse_expression ($1);
-		}
-		#$self->check_for_reload ($table) || return (-501);
-		$columns = join (',', @{ $self->{order} }) if ($columns =~ /\*/);
-		$columns =~ s/\s//g;
-		$columns =~ tr/a-z/A-Z/;
-		$self->check_columns ($columns) || return (-502);
-	
-		$values_or_error = $self->parse_columns ('select', $columns, 
-				$condition, '', \@ordercols, $descorder, 0, $distinct);    #JWT
-		return $values_or_error;
-    }
-    elsif  ($query =~ /^select\s+   #HANDLE 1 OR MORE FUNCTIONS IN SELECT. (20000306)
-			(.+)\s+
-			from\s+
-			(\w+)(.*)$/iox)
-    {
-		my ($column_stuff, $table, $extra) = ($1, $2, $3);
-#print STDERR "-select, at 3: field names only, extra=$extra=\n";
-    		my (@fields) = ();
-    		my ($fnname, $found_parin, $parincnt);
 
 		#SPLIT UP THE FIELDS BEING REQUESTED.
 
@@ -1698,15 +1672,32 @@ END_CODE
 				$ordercols[$i] =~ s/[\(\)]+//g;
 			}
 		}
-#print STDERR "-select, at 3b: extra=$extra=\n";
 		if ($extra =~ /^\s+where\s*(.+)$/i)
 		{
-#print STDERR "-select, at 3c: extra=$extra=\n";
 		    $condition = $self->parse_expression ($1);
 		}
+		if ($column_stuff =~ /\*/)
+		{
+			@fields = @{ $self->{order} };
+			$columns = join (',', @fields);
+			for (my $i=0;$i<=$#fields;$i++)
+			{
+				$fields[$i] =~ s/([^\,]+)/\$\$\_\{\U$1\E\}/g;
+			}
+		}
 		$columns =~ tr/a-z/A-Z/;
 		$self->check_columns ($columns) || return (-502);
-		$self->{use_fields} = join (',', @{ $self->{order} }[0..$#fields], );
+		if ($#fields >= 0)
+		{
+			my (@fieldnames) = @fields;
+			for (my $i=0;$i<=$#fields;$i++)
+			{
+				$fieldnames[$i] =~ s/\(.*$//;
+				$fieldnames[$i] =~ s/\$\_//;
+				$fieldnames[$i] =~ s/[^\w\,]//g;
+			}
+			$self->{use_fields} = join(',', @fieldnames);
+		}
 		$values_or_error = $self->parse_columns ('select', $columns, 
 				$condition, '', \@ordercols, $descorder, \@fields, $distinct);    #JWT
 		return $values_or_error;
@@ -1722,7 +1713,6 @@ sub update
     my ($self, $query) = @_;
     my ($i, $path, $regex, $table, $extra, $condition, $all_columns, 
 	$columns, $status);
-#print "<BR>query=$query=\n";
 	my ($psuedocols) = "CURVAL|NEXTVAL";
 
     ##++
@@ -1735,6 +1725,7 @@ sub update
 
     if ($query =~ /^update\s+($path)\s+set\s+(.+)$/io) {
 	($table, $extra) = ($1, $2);
+	return (-523)  if ($table =~ /^DUAL$/i);
 
 	#ADDED IF-STMT 20010418 TO CATCH 
 			#PARENTHESIZED SET-CLAUSES (ILLEGAL IN ORACLE & CAUSE WIERD PARSING ERRORS!)
@@ -1792,7 +1783,6 @@ sub update
 		"'$j'"/eg;
 	}
 	$extra = $expns[$#expns];    #EXTRACT WHERE-CLAUSE, IF ANY.
-#print "<BR>extra=$extra=\n";
 	$condition = ($extra =~ s/(.*)where(.+)$/where$1/i) ? $2 : '';
 	$condition =~ s/\s+//;
 	####$condition =~ s/^\((.*)\)$/$1/g;  #REMOVED 20010313 SO "WHERE ((COND) OP (COND) OP (COND)) WOULD WORK FOR DBIX-RECORDSET. (SELECT APPEARS TO WORK WITHOUT THIS).
@@ -1811,7 +1801,6 @@ sub update
 			my ($var) = $1;
 			my ($val) = $2;
 
-			$val = &pscolfn($self,$val)  if ($val =~ "$column\.$psuedocols");
 			$var =~ tr/a-z/A-Z/;
 			$columns .= $var . ',';   #ADDED 20010228.
 			$val =~ s|%\0(\d+): |pack("C",$1)|ge;
@@ -1819,7 +1808,6 @@ sub update
 			$all_columns->{$var} =~ s/\x02/\\\\/g;
 			#$all_columns->{$var} =~ s/\x03/\'\'/g;
 			$all_columns->{$var} =~ s/\x03/\'/g;   #20000108 REPL. PREV. LINE - NO NEED TO DOUBLE QUOTES (WE ESCAPE THEM) - THIS AIN'T ORACLE.
-#print "<BR>--- ac($var)=$all_columns->{$var}=\n";
 			#$all_columns->{$var} =~ s/\x04/\"\"/g;   #REMOVED 20000303.
 		!e;
 	}
@@ -2344,7 +2332,6 @@ sub insert
 {
     my ($self, $query) = @_;
     my ($i, $path, $table, $columns, $values, $status);
-#print "-INSERT: query=$query=\n";
     $path = $self->{path};
     if ($query =~ /^insert\s+into\s+                            # Keyword
                    ($path)\s*                                  # Table
@@ -2354,6 +2341,7 @@ sub insert
 {   #JWT: MAKE COLUMN LIST OPTIONAL!
 
 	($table, $columns, $values) = ($1, $2, $3);
+	return (-523)  if ($table =~ /^DUAL$/i);
 	$thefid = $table;
 	$self->check_for_reload ($table) || return (-501);
 	$columns =~ s/\s//g;
@@ -2577,7 +2565,6 @@ sub write_file
 	return 1  if $#{$self->{order}} < 0;  #ADDED 20000225 PREVENT BLANKING OUT TABLES, IE IF USER CREATES SEQUENCE W/SAME NAME AS TABLE, THEN COMMITS!
 	
 		#########$new_file =~ tr/A-Z/a-z/  unless ($self->{CaseTableNames});  #JWT:TABLE-NAMES ARE NOW CASE-INSENSITIVE!
-#print "----- FORCE-REPLACE =".$self->{sprite_forcereplace}."=\n";  #ADDED 20010912!
 	unlink ($new_file)  if ($status >= 1 && $self->{sprite_forcereplace} && -e $new_file);  #ADDED 20010912.
     if ( ($status >= 1) && (open (FILE, ">$new_file")) ) {
 	binmode FILE;   #20000404
@@ -2673,7 +2660,6 @@ sub load_database
 {
     my ($self, $file) = @_;
     my ($i, $header, @fields, $no_fields, @record, $hash, $loop, $tp, $dflt);
-
     local (*FILE);
 	local ($/) = $self->{_record};    #JWT:SUPPORT ANY RECORD-SEPARATOR!
 
@@ -2872,7 +2858,6 @@ $field .= $snippet;
 last;
 			}	
 			elsif (/^["']/) {
-				#print "Error:  Unmatched quote near ($_)!<BR>\n";
 				$self->display_error(-512);
 				return ();
 			}
@@ -2914,10 +2899,13 @@ sub chkcolumnparms   #ADDED 20001218 TO CHECK FUNCTION PARAMETERS FOR FIELD-NAME
 	#FIND EACH FIELD NAME PARAMETER & REPLACE IT WITH IT'S VALUE || NAME || EMPTY-STRING.
 
 	$evalstr =~ s/($fieldregex)/
-				$res = (defined $_->{$1}) ? $_->{$1} : $1;
+				my ($one) = $1;
+				$one =~ tr!a-z!A-Z!;
+				$res = (defined $_->{$one}) ? $_->{$one} : $one;
+
 				$res ||= '""';
 				$res;
-	/eg;
+	/eig;
 
 	$evalstr =~ s/\x04(\d+)/$strings[$1]/g;   #UNPROTECT LITERALS
 	$evalstr =~ s/\x03/\\\'/g;                #UNPROTECT QUOTES.
@@ -2930,6 +2918,11 @@ sub SYSTIME
 	return time;
 }
 
+sub SYSDATE
+{
+	return time;
+}
+
 sub NUM
 {
 	return shift;
@@ -2945,6 +2938,11 @@ sub ROWNUM
 	return (scalar (@$results) + 1);
 }
 
+sub USER
+{
+	return $sprite_user;
+}
+
 sub fn_register   #REGISTER SQL-CALLABLE FUNCTIONS.
 {
 	shift  if (ref($_[0]) eq 'HASH');   #20000224
